@@ -1,24 +1,33 @@
-"""Modulo que contiene las clases encargadas de todas las opciones que puedan tener los demas modulos"""
+"""
+Modulo que contiene las clases encargadas de todas las opciones que puedan tener los demas modulos
 
-from . import Enum, time, argparse, bcolors, utilities
+"""
+
+from .Tools import utilities, bcolors
+from . import Enum, time, argparse, os
 
 class MHType(Enum):
     """Tipos de Metaheristicas disponibles
     SA: Simulated Annealing
     GA: Genetic Algorithm
+    LS: Local Search
     """
     SA = 'SA'
     GA = 'GA'
+    LS = 'LS'
+    ILS = 'ILS'
 
 class TSPMove(Enum):
     """Tipos de movimientos disponibles para el TSP 
     TWO_OPT: Operador 2-opt
+    THREE_OPT: Operador 3-opt
     SWAP: Operador swap
     """
     TWO_OPT = 'TWO_OPT'
+    THREE_OPT = 'THREE_OPT'
     SWAP = 'SWAP'
 
-""" Simulated Annealing """
+""" S I M U L A T E D  A N N E A L I N G """
 
 class InitialSolution(Enum):
     """ Metodos disponibles para crear una solucion inicial
@@ -40,7 +49,7 @@ class CoolingType(Enum):
     LINEAR = 'LINEAR'
     LOG = 'LOG'
 
-""" Algoritmo Genetico """
+""" A L G O R I T M O   G E N E T I C O """
 
 class SelectionType(Enum):
     """Tipos de seleccion de individuos
@@ -71,6 +80,20 @@ class SelectionStrategy(Enum):
     """
     MULAMBDA = 'MULAMBDA'
     MUPLUSLAMBDA = 'MUPLUSLAMBDA'
+    
+""" L O C A L  S E A R C H  E  I T E R A T E D  L O C A L  S E A R C H """  
+    
+class PerturbationType(Enum):
+    """Tipos de movimientos de perturbacion para Iterated Local Search
+    TWO_OPT: Operador 2-opt
+    THREE_OPT: Operador 3-opt
+    SWAP: Operador swap
+    RANDOM: Operador aleatorio entre los anteriores
+    """
+    TWO_OPT = 'TWO_OPT'
+    THREE_OPT = 'THREE_OPT'
+    SWAP = 'SWAP'
+    RANDOM = 'RANDOM'
 
 class AlgorithmsOptions():
     """
@@ -132,7 +155,7 @@ class AlgorithmsOptions():
         Mostrar las opciones y parametros finales
     """
     
-    # OPCIONES GENERALES
+    """ O P C I O N E S  G E N E R A L E S """
     
     solution = "solution.txt"	# Archivo para imprimir la solucion
 
@@ -144,7 +167,7 @@ class AlgorithmsOptions():
      
     metaheuristic = MHType.SA # Tipo de metaheuristica a ejecutar   
      
-    move = TSPMove.SWAP # Tipo del movimiento para la metaheuristica
+    move = TSPMove.TWO_OPT # Tipo del movimiento para la metaheuristica o bien tipo de busqueda Local Search
     
     max_evaluations = 1000 # Evaluaciones maximas
     
@@ -152,7 +175,7 @@ class AlgorithmsOptions():
     
     max_time = 60.0 # Tiempo de ejecucion maximo
     
-    initial_solution = InitialSolution.DETERMINISTIC # Solucion Inicial
+    initial_solution = InitialSolution.RANDOM # Solucion Inicial
     
     silent = False # Modo silencioso
 
@@ -160,7 +183,11 @@ class AlgorithmsOptions():
 
     replit = False # Si se esta ejecutando en Replit.com ya que causa inconveniencia con la graficacion
 
-    # OPCIONES PARA SIMULATED ANNEALING 
+    verbose = False # modo verbose
+    
+    gui = False # modo Interfaz grafica
+    
+    """ O P C I O N E S  P A R A  S I M U L A T E D  A N N E A L I N G """
     
     alpha = 0.98 # Parametro alfa para el enfriamiento
     
@@ -170,7 +197,7 @@ class AlgorithmsOptions():
     
     cooling = CoolingType.GEOMETRIC # Tipo de enfriamiento
 
-    # OPCIONES PARA ALGORITMO GENETICO 
+    """ O P C I O N E S   P A R A   A L G O R I T M O   G E N E T I C O """
     
     pop_size = 10 # Cantidad de individuos de la poblacion 
 	
@@ -187,6 +214,14 @@ class AlgorithmsOptions():
     selection_strategy = SelectionStrategy.MULAMBDA # Estrategia de seleccion de la nueva poblacion
     
     gselection_type = SelectionType.RANDOM # Seleccion de la nueva poblacion
+    
+    """ O P C I O N E S   P A R A   L O C A L  S E A R C H  E  I T E R A T E D  L O C A L  S E A R C H """
+    
+    perturbation = PerturbationType.SWAP
+    
+    bestImprovement = False
+    
+    nPerturbations = 3
 
     def __init__(self, argv=[], **kwargs) -> None:
 
@@ -199,7 +234,7 @@ class AlgorithmsOptions():
         # definir semilla para el generador aleatorio
         utilities.random.seed(self.seed)
         # Mostrar Opciones 
-        self.printOptions()
+        #self.printOptions()
     
 
     def readOptions(self, argv: list, kwargs: dict) -> None:
@@ -216,13 +251,16 @@ class AlgorithmsOptions():
         parser.add_argument("-s", "--silent", help="Ejecuta sin mostrar los cambios en cada ciclo de los algoritmos", action="store_true")
         parser.add_argument("-vi", "--visualize", help="Muestra la visualizacion de trayectoria a la mejor solucion de forma grafica", action="store_true")
         parser.add_argument("-re", "--replit", help="Si se ejecuta en Replit.com ya que la visualizacion puede tener inconvenientes", action="store_true")
+        parser.add_argument("-v", "--verbose", help="Ejecuta mostrando todos los detalles de cada iteracion", action="store_true")
+        parser.add_argument("-gui", "--gui", help="Ejecuta en modo interfaz grafica", action="store_true")
 
-        parser.add_argument("-mh", "--metaheuristic", help="Tipo de Metaherisitica a usar:\n SA: Simulated Annealing\n GA: Genetic Algorithm")
+                
+        parser.add_argument("-mh", "--metaheuristic", help="Tipo de Metaherisitica a usar:\n SA: Simulated Annealing\n GA: Genetic Algorithm\n LS: Local Search")
+        parser.add_argument("-al", "--algorithm", help="Tipo de Algoritmo a usar:\n SA: Simulated Annealing\n GA: Genetic Algorithm\n LS: Local Search")
         parser.add_argument("-i", "--instance", help="Archivo con la instancia a utilizar en formato TSPLIB")
         parser.add_argument("-se", "--seed", help="Numero para ser usado como semilla para el generador de numeros aleatorios")
-        parser.add_argument("-sol", "--solution", help="Nombre del archivo de salida para la solucion")
-        parser.add_argument("-tra", "--trajectory", help="Nombre del archivo de salida para la trayectoria de la solucion")
-        parser.add_argument("-mhm", "--move", help="Tipo de movimiento a utilizar en la heuristica [ 2opt | swap ]")
+        parser.add_argument("-sol", "--solution", help="Nombre del archivo de salida para la solucion y trayectoria")
+        parser.add_argument("-mhm", "--move", help="Tipo de movimiento a utilizar en la heuristica [ 2opt | swap | 3opt ]")
         parser.add_argument("-e", "--evaluations", help="Numero maximo de soluciones a evaluar")
         parser.add_argument("-it", "--iterations", help="Numero maximo de iteraciones a realizar")
         parser.add_argument("-t", "--time", help="Limite de tiempo de ejecucion en segundos")
@@ -239,10 +277,16 @@ class AlgorithmsOptions():
         parser.add_argument("-o", "--osize", help="Cantidad de hijos a generar ]0,INT_MAX]")
         parser.add_argument("-ps", "--pselection", help="Operador de seleccion de padres [ random | best | roulette | tournament ]")
         parser.add_argument("-cr", "--crossover", help="Operador de crossover [ ox | opx | pmx ]")
-        parser.add_argument("-mu", "--mutation", help="Operador de mutacion [ swap | 2opt ]")
+        parser.add_argument("-mu", "--mutation", help="Operador de mutacion [ swap | 2opt | 3opt ]")
         parser.add_argument("-mp", "--mprobability", help="Probabilidad de mutacion [0.0,1.0]")
         parser.add_argument("-gs", "--gselection", help="Operador de seleccion de poblacion [ random | best | roulette | tournament ]")
         parser.add_argument("-g", "--gstrategy", help="Estrategia de seleccion de padres [ mu,lambda | mu+lambda ]")
+        
+        # Definir argumentos de Local Search e Iterated Local Search
+        parser.add_argument("-b", "--best", help="Ejecuta Local Search en modo best improvement", action="store_true")
+        parser.add_argument("-per", "--perturbation", help="Tipo de perturbacion a aplicar en ITS [ 2opt | swap | 3opt ]")
+        parser.add_argument("-np", "--nperturbations", help="Cantidad de perturbaciones a aplicar en una iteracion ]0,INT_MAX]")
+        
         # Procesar argumentos
         args = parser.parse_args()
 
@@ -253,12 +297,17 @@ class AlgorithmsOptions():
             # Procesar argumentos de Simulated Annealing
             self.argsSA(args, kwargs)
             # Validar logica de opciones
-            self.validateSA()
+            if self.errorsSA():
+                exit()
         elif self.metaheuristic == MHType.GA:
             # Procesar argumentos de Algoritmo Genetico
             self.argsGA(args, kwargs) 
             # Validar logica de opciones
-            self.validateGA() 
+            if self.errorsGA():
+                exit()
+        elif self.metaheuristic == MHType.LS or self.metaheuristic == MHType.ILS:
+            # Procesar argumentos de Local Search e Iterated Local Search
+            self.argsLS(args, kwargs)
         
 
     def argsGeneral(self, args: argparse.Namespace, kwargs: dict) -> None:
@@ -277,21 +326,30 @@ class AlgorithmsOptions():
             except: 
                 print(f"{bcolors.FAIL}Error: La semilla debe ser un numero entero (-s | --seed){bcolors.ENDC}")
 
-        # Archivo de salida para solucion
-        if (args.solution or 'output' in kwargs):
+        # Archivo de salida para solucion y trayectoria
+        if (args.solution or 'solution' in kwargs):
             self.solution = args.solution if args.solution else kwargs['solution']
-            if not '.txt' in self.solution:
+            self.trajectory = args.solution if args.solution else kwargs['solution']
+            
+            check = os.path.splitext(self.solution) # separa la ruta de la extension en lista
+            if not check[1]:
                 self.solution += '.txt'
-
-        # Archivo de salida para trayectoria
-        if (args.trajectory or 'trajectory' in kwargs):
-            self.trajectory = args.trajectory if args.trajectory else kwargs['trajectory']
-            if not '.csv' in self.trajectory:
+                
+            check = os.path.splitext(self.trajectory)
+            if not check[1]:
                 self.trajectory += '.csv'
+                
+        # Si se activa modo verbose
+        if (args.verbose or 'verbose' in kwargs):
+            self.verbose = args.verbose if args.verbose else kwargs['verbose']
 
         # Si se visualizara la trayectoria
-        if (args.visualize or 'graphic' in kwargs):
+        if (args.visualize or 'visualize' in kwargs):
             self.visualize = args.visualize if args.visualize else kwargs['visualize']
+        
+        # Si se activa el modo interfaz grafica
+        if (args.gui or 'gui' in kwargs):
+            self.gui = args.gui if args.gui else kwargs['gui']
 
         # Si se ejecuta en Replit.com
         if (args.replit or 'replit' in kwargs):
@@ -308,6 +366,10 @@ class AlgorithmsOptions():
                 self.metaheuristic = MHType.SA
             elif (val == 'GA'):
                 self.metaheuristic = MHType.GA
+            elif (val == 'LS'):
+                self.metaheuristic = MHType.LS
+            elif (val == 'ILS'):
+                self.metaheuristic = MHType.ILS
             else: print(f"{bcolors.FAIL}Error: Metaheuristica no reconocida (-mh | --metaheristic) {bcolors.ENDC}")  
         
         # Numero maximo de evaluaciones
@@ -329,17 +391,12 @@ class AlgorithmsOptions():
             val = args.move.lower() if args.move else kwargs['move'].lower()
             if (val == '2opt' or val == '2-opt'):
                 self.move = TSPMove.TWO_OPT
+            elif (val == '3opt' or val == '3-opt'):
+                self.move = TSPMove.THREE_OPT
             elif (val == 'swap'):
                 self.move = TSPMove.SWAP
             else: print(f"{bcolors.FAIL}Error: Tipo de movimiento no reconocido (-mhm | --move) {bcolors.ENDC}") 
-
-        # Modo de salida reducido para no mostrar todos los cambios en los ciclos de los algoritmos
-        if (args.silent):
-            self.silent = True
-
-
-    def argsSA(self, args: argparse.Namespace, kwargs: dict) -> None:
-        """Procesar los argumentos de Simulated Annealing"""
+            
         # Solucion inicial
         if (args.insol or 'insol' in kwargs):
             val = args.insol.upper() if args.insol else kwargs['insol'].upper()
@@ -351,6 +408,14 @@ class AlgorithmsOptions():
                 self.initial_solution = InitialSolution.DETERMINISTIC
             else: print(f"{bcolors.FAIL}Error: Opcion no reconocida en solucion inicial (-is | --inso) {bcolors.ENDC}")
 
+        # Modo de salida reducido para no mostrar todos los cambios en los ciclos de los algoritmos
+        if (args.silent):
+            self.silent = True
+
+
+    def argsSA(self, args: argparse.Namespace, kwargs: dict) -> None:
+        """Procesar los argumentos de Simulated Annealing"""
+
         # Seleccion del esquema de enfriamiento
         if (args.cooling or 'cooling' in kwargs):
             val = args.cooling.lower() if args.cooling else kwargs['cooling'].lower()
@@ -429,6 +494,8 @@ class AlgorithmsOptions():
             val = args.mutation.lower() if args.mutation else kwargs['mutation'].lower()
             if (val == '2opt' or val == '2-opt'):
                 self.mutation_type = TSPMove.TWO_OPT
+            elif (val == '3opt' or val == '3-opt'):
+                self.mutation_type = TSPMove.THREE_OPT
             elif (val == 'swap'):
                 self.mutation_type = TSPMove.SWAP
             else: print(f"{bcolors.FAIL}Error: Tipo de mutacion no reconocido (-mu | --mutation) {bcolors.ENDC}")
@@ -461,9 +528,37 @@ class AlgorithmsOptions():
             elif (val == 'mu+lambda'):
                 self.selection_strategy = SelectionStrategy.MUPLUSLAMBDA
             else: print(f"{bcolors.FAIL}Error: Tipo de seleccion de padres no reconocido (-g | --gstrategy) {bcolors.ENDC}")
+            
+            
+    def argsLS(self, args: argparse.Namespace, kwargs: dict) -> None:
+        """Procesar argumentos de Local Search"""
+        
+        # Seleccion del movimiento para la metaheuristica
+        if (args.perturbation or 'perturbation' in kwargs):
+            val = args.perturbation.lower() if args.move else kwargs['perturbation'].lower()
+            if (val == '2opt' or val == '2-opt'):
+                self.perturbation = PerturbationType.TWO_OPT
+            elif (val == '3opt' or val == '3-opt'):
+                self.perturbation = PerturbationType.THREE_OPT
+            elif (val == 'swap'):
+                self.perturbation = PerturbationType.SWAP
+            elif (val == 'random'):
+                self.perturbation = PerturbationType.RANDOM
+            else: print(f"{bcolors.FAIL}Error: Tipo de perturbacion no reconocido (-per | --perturbation) {bcolors.ENDC}")
+        
+        # Si se ejecuta en Replit.com
+        if (args.best or 'best' in kwargs):
+            self.bestImprovement = args.best if args.best else kwargs['best']
+        
+        # Cantidad de hijos a generar
+        if (args.nperturbations or 'nperturbations' in kwargs):
+            try:
+                self.nPerturbations = int(args.nperturbations) if args.nperturbations else int(kwargs['nperturbations'])
+            except: 
+                print(f"{bcolors.FAIL}Error: El numero de perturbaciones debe ser un numero entero (-np | --nperturbations){bcolors.ENDC}")
 
 
-    def validateSA(self) -> None:
+    def errorsSA(self) -> bool:
         """ Validar que algunos parametros cumplan con la logica del algoritmo a aplicar """
         error = False
         if (self.max_evaluations <= 0 or self.max_iterations <= 0):
@@ -478,10 +573,9 @@ class AlgorithmsOptions():
         if (self.t0 <= self.tmin):
             print(f"{bcolors.FAIL}Error: t0 debe ser > tmin, valor tmin: {self.tmin} valor t0: {self.t0} (-t0 | --tini y -tm | --tmin){bcolors.ENDC}")
             error = True
-        if error:
-            exit()
+        return error
     
-    def validateGA(self) -> None:
+    def errorsGA(self) -> bool:
         """ Validar que algunos parametros cumplan con la logica del algoritmo a aplicar """
         error = False
         if (self.max_evaluations <= 0 or self.max_iterations <= 0):
@@ -495,10 +589,19 @@ class AlgorithmsOptions():
             error = True
         if (self.pop_size <= 1):
             print(f"{bcolors.FAIL}Error: La cantidad de individuos de la poblacion (-p | --psize) debe ser > 1{bcolors.ENDC}")
-            error = True
-        if error:
-            exit()
-                   
+            error = True     
+        return error
+    
+    
+    def errors(self) -> bool:
+        """ Devuelve verdadero si hay errores en las configuraciones del metodo de busqueda """
+        if self.metaheuristic == MHType.SA:
+            return self.errorsSA()
+        elif self.metaheuristic == MHType.GA:
+            return self.errorsGA()
+        return False
+    
+    
     def printOptions(self) -> None:
         """ Mostrar las opciones y parametros finales """
         # Opciones generales
@@ -530,5 +633,13 @@ class AlgorithmsOptions():
             print(f"{bcolors.OKBLUE}Probabilidad de mutacion: {bcolors.ENDC}{self.mutation_prob}")
             print(f"{bcolors.OKBLUE}Estrategia de seleccion para las nuevas poblaciones: {bcolors.ENDC}{self.selection_strategy.value}")
             print(f"{bcolors.OKBLUE}Tipo de seleccion de la nueva poblacion: {bcolors.ENDC}{self.gselection_type.value}")
+        elif (self.metaheuristic == MHType.LS or self.metaheuristic == MHType.ILS):
+            print(f"{bcolors.HEADER}\n\t\tOPCIONES PARA LOCAL SEARCH E ITERATED LOCAL SEARCH\n {bcolors.ENDC}")        
+            print(f"{bcolors.OKBLUE}Tipo de movimiento para busqueda: {bcolors.ENDC}{self.move.value}")
+            print(f"{bcolors.OKBLUE}Best Improvement: {bcolors.ENDC}{self.bestImprovement}")
+            print(f"{bcolors.OKBLUE}Tipo de perturbacion para busqueda ILS: {bcolors.ENDC}{self.perturbation.value}")
+            print(f"{bcolors.OKBLUE}Numero de perturbaciones a aplicar para busqueda ILS: {bcolors.ENDC}{self.nPerturbations}")
+        
             
+                        
         print()
\ No newline at end of file
