@@ -1,7 +1,11 @@
-"""Modulo principal que utiliza todas las demas clases para ejecutar el framework"""
+"""
+Modulo principal que utiliza todas las demas clases para ejecutar el framework
 
-from .Algorithms import GeneticAlgorithm, SimulatedAnnealing, timer
-from . import sys, os, AlgorithmsOptions, MHType, Tsp, Tour, bcolors
+"""
+
+from .Algorithms import GeneticAlgorithm, SimulatedAnnealing, LocalSearch, IteratedLocalSearch, timer
+from .Tools import bcolors, gui
+from . import sys, os, AlgorithmsOptions, MHType, Tsp, Tour
 
 def main(argv=sys.argv) -> None:
     """
@@ -15,10 +19,17 @@ def main(argv=sys.argv) -> None:
     start = timer() # tiempo inicial de ejecucion
     # leer e inicializar las opciones 
     options = AlgorithmsOptions(argv=argv)
+    
+    # Si se esta en modo GUI
+    if options.gui:
+        gui.main(options)
+        return
+        
+    # Mostrar Opciones 
+    options.printOptions()
 
     # leer e interpretar el problema TSP leido desde la instancia definida
     problem = Tsp(filename=options.instance)
-    #print(problema.random_tour())
 
     # Ejecutar Metaheuristica Simulated Annealing
     if (options.metaheuristic == MHType.SA):
@@ -36,6 +47,22 @@ def main(argv=sys.argv) -> None:
         solver = GeneticAlgorithm(options=options, problem=problem)
         # Ejecutar la busqueda
         solver.search()
+        
+    elif (options.metaheuristic == MHType.LS):
+        # Solucion inicial
+        first_solution = Tour(type_initial_sol=options.initial_solution, problem=problem)
+        # Crear solver
+        solver = LocalSearch(options=options, problem=problem)
+        # Ejecutar la busqueda
+        solver.search(first_solution)
+        
+    elif (options.metaheuristic == MHType.ILS):
+        # Solucion inicial
+        first_solution = Tour(type_initial_sol=options.initial_solution, problem=problem)
+        # Crear solver
+        solver = IteratedLocalSearch(options=options, problem=problem)
+        # Ejecutar la busqueda
+        solver.search(first_solution)
 
     else: 
         # Crear solver
@@ -53,8 +80,5 @@ def main(argv=sys.argv) -> None:
     print(f"{bcolors.BOLD}Tiempo total de ejecucion: {bcolors.ENDC}{bcolors.OKBLUE} {end-start:.3f} segundos{bcolors.ENDC}")
     
     if options.visualize:
-        solver.visualize(options.replit)
- 
-   
-if __name__ == '__main__':
-    main()
\ No newline at end of file
+        solver.visualize()
+ 
\ No newline at end of file
